package exe

import "git.urbach.dev/cli/q/src/config"

// Executable is a generic definition of the binary that later gets translated to OS-specific formats.
type Executable []*Section

// MakeSections creates new sections.
func MakeSections(headerEnd int, raw ...[]byte) Executable {
	sections := make(Executable, len(raw))

	for i, data := range raw {
		sections[i] = &Section{Bytes: data}
	}

	sections.Update(headerEnd)
	return sections
}

// Update recalculates all section offsets.
func (sections Executable) Update(headerEnd int) {
	first := sections[0]
	first.FileOffset, first.Padding = AlignPad(headerEnd, config.FileAlign)
	first.MemoryOffset = Align(headerEnd, config.MemoryAlign)

	for i, section := range sections[1:] {
		previous := sections[i]
		section.FileOffset, section.Padding = AlignPad(previous.FileOffset+len(previous.Bytes), config.FileAlign)
		section.MemoryOffset = Align(previous.MemoryOffset+len(previous.Bytes), config.MemoryAlign)
	}
}
